{
  "Records": [
    {
      "Ckafka": {
        "topic": "demo-topic",
        "partition":"",
        "offset":1,
        "msgKey": "demo",
        "msgBody": "Hello from ckafka"
      }
    }
  ]
}