{
  "Records": [
    {
      "CMQ": {
        "type": "topic",
        "topicOwner": {{owner}},
        "topicName": "demoTopic",
        "subscriptionName":"demoSub",
        "publishTime": "2018-12-25T11:01:00Z",
        "msgId": "123345346123123123",
        "requestId":"123345346123123",
        "msgBody": "Hello from CMQ!",
        "msgTag": "tag1,tag2"
      }
    }
  ]
}